@@ -9,144 +9,112 @@ use File::Spec;
 use Scalar::Util qw( blessed );
 
 use overload
-    '""' => \&as_string,
+    '""'     => \&as_string,
     fallback => 1;
 
-our $VERSION = '1.21';
-
-
-sub new
-{
+sub new {
     my $class = shift;
-    my %p = @_;
+    my %p     = @_;
 
     # Backwards compatibility - this parameter was renamed to no_refs
     # ages ago.
     $p{no_refs} = delete $p{no_object_refs}
         if exists $p{no_object_refs};
 
-    my $self =
-        bless { index  => undef,
-                frames => [],
-                raw    => [],
-                %p,
-              }, $class;
+    my $self = bless {
+        index  => undef,
+        frames => [],
+        raw    => [],
+        %p,
+    }, $class;
 
     $self->_record_caller_data();
 
     return $self;
 }
 
-sub _record_caller_data
-{
+sub _record_caller_data {
     my $self = shift;
 
     # We exclude this method by starting one frame back.
     my $x = 1;
-    while ( my @c =
-            do { package DB; @DB::args = (); caller($x++) } )
-    {
-        my @a = @DB::args;
+    while (
+        my @c
+        = do { package DB; @DB::args = (); caller( $x++ ) }
+        ) {
+        my @args = @DB::args;
 
         if ( $self->{no_refs} ) {
-            @a = map { ref $_ ? $self->_ref_to_string($_) : $_ } @a;
+            @args = map { ref $_ ? $self->_ref_to_string($_) : $_ } @args;
         }
 
-        push @{ $self->{raw} },
-            { caller => \@c,
-              args   => \@a,
+        push @{ $self->{raw} }, {
+            caller => \@c,
+            args   => \@args,
             };
     }
 }
 
-sub _ref_to_string
-{
+sub _ref_to_string {
     my $self = shift;
     my $ref  = shift;
 
-    return overload::StrVal($ref)
+    return overload::AddrRef($ref)
         if blessed $ref && $ref->isa('Exception::Class::Base');
 
-    return overload::StrVal($ref) unless $self->{respect_overload};
+    return overload::AddrRef($ref) unless $self->{respect_overload};
 
     local $@;
     local $SIG{__DIE__};
 
     my $str = eval { $ref . '' };
 
-    return $@ ? overload::StrVal($ref) : $str;
+    return $@ ? overload::AddrRef($ref) : $str;
 }
 
-sub _ecb_hack
-{
-    my $self = shift;
-    my $ref  = shift;
-
-    # This avoids a loop between Exception::Class::Base and this module
-    if ( $ref->can('show_trace') )
-    {
-        my $t = $ref->show_trace;
-        $ref->show_trace(0);
-        my $s = "$ref";
-        $ref->show_trace($t);
-
-        return $s;
-    }
-    else
-    {
-        # hack but should work with older
-        # versions of E::C::B
-        return $ref->{message};
-    }
-}
-
-sub _make_frames
-{
+sub _make_frames {
     my $self = shift;
 
     my $filter = $self->_make_frame_filter;
 
     my $raw = delete $self->{raw};
-    for my $r ( @{$raw} )
-    {
+    for my $r ( @{$raw} ) {
         next unless $filter->($r);
 
         $self->_add_frame( $r->{caller}, $r->{args} );
     }
 }
 
-my $default_filter = sub { 1 };
-sub _make_frame_filter
-{
+my $default_filter = sub {1};
+
+sub _make_frame_filter {
     my $self = shift;
 
-    my (@i_pack_re, %i_class);
-    if ( $self->{ignore_package} )
-    {
-        $self->{ignore_package} =
-            [ $self->{ignore_package} ] unless UNIVERSAL::isa( $self->{ignore_package}, 'ARRAY' );
+    my ( @i_pack_re, %i_class );
+    if ( $self->{ignore_package} ) {
+        $self->{ignore_package} = [ $self->{ignore_package} ]
+            unless UNIVERSAL::isa( $self->{ignore_package}, 'ARRAY' );
 
-        @i_pack_re = map { ref $_ ? $_ : qr/^\Q$_\E$/ } @{ $self->{ignore_package} };
+        @i_pack_re
+            = map { ref $_ ? $_ : qr/^\Q$_\E$/ } @{ $self->{ignore_package} };
     }
 
     my $p = __PACKAGE__;
     push @i_pack_re, qr/^\Q$p\E$/;
 
-    if ( $self->{ignore_class} )
-    {
-        $self->{ignore_class} = [ $self->{ignore_class} ] unless ref $self->{ignore_class};
-        %i_class = map {$_ => 1} @{ $self->{ignore_class} };
+    if ( $self->{ignore_class} ) {
+        $self->{ignore_class} = [ $self->{ignore_class} ]
+            unless ref $self->{ignore_class};
+        %i_class = map { $_ => 1 } @{ $self->{ignore_class} };
     }
 
     my $user_filter = $self->{frame_filter};
 
-    return sub
-    {
+    return sub {
         return 0 if grep { $_[0]{caller}[0] =~ /$_/ } @i_pack_re;
         return 0 if grep { $_[0]{caller}[0]->isa($_) } keys %i_class;
 
-        if ( $user_filter )
-        {
+        if ($user_filter) {
             return $user_filter->( $_[0] );
         }
 
@@ -154,45 +122,45 @@ sub _make_frame_filter
     };
 }
 
-sub _add_frame
-{
+sub _add_frame {
     my $self = shift;
     my $c    = shift;
     my $args = shift;
 
     # eval and is_require are only returned when applicable under 5.00503.
-    push @$c, (undef, undef) if scalar @$c == 6;
+    push @$c, ( undef, undef ) if scalar @$c == 6;
 
-    if ( $self->{no_refs} )
-    {
+    if ( $self->{no_refs} ) {
     }
 
     push @{ $self->{frames} },
-        Devel::StackTraceFrame->new( $c, $args,
-                                     $self->{respect_overload}, $self->{max_arg_length} );
+        Devel::StackTraceFrame->new(
+        $c,
+        $args,
+        $self->{respect_overload},
+        $self->{max_arg_length},
+        $self->{message},
+        $self->{indent}
+        );
 }
 
-sub next_frame
-{
+sub next_frame {
     my $self = shift;
 
     # reset to top if necessary.
     $self->{index} = -1 unless defined $self->{index};
 
     my @f = $self->frames();
-    if ( defined $f[ $self->{index} + 1 ] )
-    {
+    if ( defined $f[ $self->{index} + 1 ] ) {
         return $f[ ++$self->{index} ];
     }
-    else
-    {
+    else {
         $self->{index} = undef;
         return undef;
     }
 }
 
-sub prev_frame
-{
+sub prev_frame {
     my $self = shift;
 
     my @f = $self->frames();
@@ -200,26 +168,22 @@ sub prev_frame
     # reset to top if necessary.
     $self->{index} = scalar @f unless defined $self->{index};
 
-    if ( defined $f[ $self->{index} - 1 ] && $self->{index} >= 1 )
-    {
+    if ( defined $f[ $self->{index} - 1 ] && $self->{index} >= 1 ) {
         return $f[ --$self->{index} ];
     }
-    else
-    {
+    else {
         $self->{index} = undef;
         return undef;
     }
 }
 
-sub reset_pointer
-{
+sub reset_pointer {
     my $self = shift;
 
     $self->{index} = undef;
 }
 
-sub frames
-{
+sub frames {
     my $self = shift;
 
     $self->_make_frames() if $self->{raw};
@@ -227,31 +191,27 @@ sub frames
     return @{ $self->{frames} };
 }
 
-sub frame
-{
+sub frame {
     my $self = shift;
-    my $i = shift;
+    my $i    = shift;
 
     return unless defined $i;
 
     return ( $self->frames() )[$i];
 }
 
-sub frame_count
-{
+sub frame_count {
     my $self = shift;
 
-    return scalar ( $self->frames() );
+    return scalar( $self->frames() );
 }
 
-sub as_string
-{
+sub as_string {
     my $self = shift;
 
-    my $st = '';
+    my $st    = '';
     my $first = 1;
-    foreach my $f ( $self->frames() )
-    {
+    foreach my $f ( $self->frames() ) {
         $st .= $f->as_string($first) . "\n";
         $first = 0;
     }
@@ -260,8 +220,7 @@ sub as_string
 }
 
 # Hide from PAUSE
-package
-    Devel::StackTraceFrame;
+package Devel::StackTraceFrame;
 
 use strict;
 use warnings;
@@ -269,81 +228,82 @@ use warnings;
 our $VERSION = $Devel::StackTrace::VERSION;
 
 # Create accessor routines
-BEGIN
-{
+BEGIN {
     no strict 'refs';
-    foreach my $f ( qw( package filename line subroutine hasargs
-                        wantarray evaltext is_require hints bitmask args ) )
-    {
+    foreach my $f (
+        qw( package filename line subroutine hasargs
+        wantarray evaltext is_require hints bitmask args )
+        ) {
         next if $f eq 'args';
         *{$f} = sub { my $s = shift; return $s->{$f} };
     }
 }
 
 {
-    my @fields =
-        ( qw( package filename line subroutine hasargs wantarray
-              evaltext is_require hints bitmask ) );
+    my @fields = (
+        qw( package filename line subroutine hasargs wantarray
+            evaltext is_require hints bitmask )
+    );
 
-    sub new
-    {
+    sub new {
         my $proto = shift;
         my $class = ref $proto || $proto;
 
         my $self = bless {}, $class;
 
-        @{ $self }{ @fields } = @{$_[0]};
+        @{$self}{@fields} = @{ shift() };
 
         # fixup unix-style paths on win32
         $self->{filename} = File::Spec->canonpath( $self->{filename} );
 
-        $self->{args} = $_[1];
+        $self->{args} = shift;
 
-        $self->{respect_overload} = $_[2];
+        $self->{respect_overload} = shift;
 
-        $self->{max_arg_length} = $_[3];
+        $self->{max_arg_length} = shift;
+
+        $self->{message} = shift;
+
+        $self->{indent} = shift;
 
         return $self;
     }
 }
 
-sub args
-{
+sub args {
     my $self = shift;
 
     return @{ $self->{args} };
 }
 
-sub as_string
-{
-    my $self = shift;
+sub as_string {
+    my $self  = shift;
     my $first = shift;
 
     my $sub = $self->subroutine;
+
     # This code stolen straight from Carp.pm and then tweaked.  All
     # errors are probably my fault  -dave
-    if ($first)
-    {
-        $sub = 'Trace begun';
+    if ($first) {
+        $sub
+            = defined $self->{message}
+            ? $self->{message}
+            : 'Trace begun';
     }
-    else
-    {
+    else {
+
         # Build a string, $sub, which names the sub-routine called.
         # This may also be "require ...", "eval '...' or "eval {...}"
-        if (my $eval = $self->evaltext)
-        {
-            if ($self->is_require)
-            {
+        if ( my $eval = $self->evaltext ) {
+            if ( $self->is_require ) {
                 $sub = "require $eval";
             }
-            else
-            {
+            else {
                 $eval =~ s/([\\\'])/\\$1/g;
                 $sub = "eval '$eval'";
             }
         }
-        elsif ($sub eq '(eval)')
-        {
+        elsif ( $sub eq '(eval)' ) {
             $sub = 'eval {...}';
         }
 
@@ -353,10 +313,9 @@ sub as_string
         #
         # We copy them because they're going to be modified.
         #
-        if ( my @a = $self->args )
-        {
-            for (@a)
-            {
+        if ( my @a = $self->args ) {
+            for (@a) {
+
                 # set args to the string "undef" if undefined
                 $_ = "undef", next unless defined $_;
 
@@ -364,11 +323,9 @@ sub as_string
                 $_ = $self->Devel::StackTrace::_ref_to_string($_)
                     if ref $_;
 
-                eval
-                {
+                eval {
                     if ( $self->{max_arg_length}
-                         && length $_ > $self->{max_arg_length} )
-                    {
+                        && length $_ > $self->{max_arg_length} ) {
                         substr( $_, $self->{max_arg_length} ) = '...';
                     }
 
@@ -382,30 +339,39 @@ sub as_string
                     s/([\0-\37\177])/sprintf("^%c",ord($1)^64)/eg;
                 };
 
-                if ( my $e = $@ )
-                {
+                if ( my $e = $@ ) {
                     $_ = $e =~ /malformed utf-8/i ? '(bad utf-8)' : '?';
                 }
             }
 
             # append ('all', 'the', 'arguments') to the $sub string
-            $sub .= '(' . join(', ', @a) . ')';
+            $sub .= '(' . join( ', ', @a ) . ')';
             $sub .= ' called';
         }
     }
 
-    return "$sub at " . $self->filename . ' line ' . $self->line;
+    # If the user opted into indentation (a la Carp::confess), pre-add a tab
+    my $tab = $self->{indent} && !$first ? "\t" : q{};
+
+    return "${tab}$sub at " . $self->filename . ' line ' . $self->line;
 }
 
 1;
 
+# ABSTRACT: Stack trace and stack trace frame objects
 
-__END__
+
+
+=pod
 
 =head1 NAME
 
 Devel::StackTrace - Stack trace and stack trace frame objects
 
+=head1 VERSION
+
+version 1.23
+
 =head1 SYNOPSIS
 
   use Devel::StackTrace;
@@ -515,7 +481,7 @@ representation.
 
 =item * respect_overload => $boolean
 
-By default, Devel::StackTrace will call C<overload::StrVal()> to get
+By default, Devel::StackTrace will call C<overload::AddrRef()> to get
 the underlying string representation of an object, instead of
 respecting the object's stringification overloading.  If you would
 prefer to see the overloaded representation of objects in stack
@@ -528,6 +494,17 @@ each subroutine call. Setting this parameter causes it to truncate the
 argument's string representation if it is longer than this number of
 characters.
 
+=item * message => $string
+
+By default, Devel::StackTrace will use 'Trace begun' as the message for the
+first stack frame when you call C<as_string>. You can supply an alternative
+message using this option.
+
+=item * indent => $boolean
+
+If this parameter is true, each stack frame after the first will start with a
+tab character, just like C<Carp::confess()>.
+
 =back
 
 =item * $trace->next_frame
@@ -620,15 +597,18 @@ or via email at bug-devel-stacktrace@rt.cpan.org.
 
 =head1 AUTHOR
 
-Dave Rolsky, <autarch@urth.org>
+  Dave Rolsky <autarch@urth.org>
 
-=head1 COPYRIGHT
+=head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2000-2006 David Rolsky.  All rights reserved.  This
-program is free software; you can redistribute it and/or modify it
-under the same terms as Perl itself.
+This software is Copyright (c) 2010 by Dave Rolsky.
 
-The full text of the license can be found in the LICENSE file included
-with this module.
+This is free software, licensed under:
+
+  The Artistic License 2.0
 
 =cut
+
+
+__END__
+
